#include "CBissCard.h"

CBissCard::CBissCard()
{
	for (int i = 0; i<3; i++)
	{
		vMemAddr[i] = 0;
	}
	PciData = (PPCI_COMMON_CONFIG)buffer;//PCI configuration
	m_pdwPhyMemory = 0;
	m_llMemoryAddr.QuadPart = 0;
	m_iDataGroupNum = 1;
	m_dwCardNum = 0;
	m_clock_freq = PCIE_2011_CLOCK_2M;
	m_protocol = BISS_C;
	for (int i = 0; i < 5; i++)
	{
		m_clock_bit[i] = 26;
	}
	m_ppsBissInfo = new __PACKET*[m_iDataGroupNum];
	for (int i = 0; i < m_iDataGroupNum; i++)
	{
		m_ppsBissInfo[i] = new __PACKET[5];
	}
}

CBissCard::~CBissCard()
{
	if (m_ppsBissInfo != NULL)
	{
		delete[]m_ppsBissInfo;
		m_ppsBissInfo = NULL;
	}
}

int CBissCard::SearchDeviceBiss(int nCardNumber, PCI_SLOT_NUMBER *pSlotNumber, PPCI_COMMON_CONFIG PciData)
{
	ULONG   dwDeviceNumber;      // logical slot number for the PCI adapter
	ULONG   dwFunctionNumber;    // function number on the specified adapter
	ULONG   dwBytesWritten;      // return value from RtGetBusDataByOffset
	ULONG   dwBus;               // bus number
	BOOLEAN Flag = TRUE;       // TRUE as long as we have PCI busses           
	int nCardIndex = 0;

	pSlotNumber->u.bits.Reserved = 0;

	// 
	// All PCI cards have been searched when we are out of PCI busses. (Flag = 0) 
	//
	for (dwBus = 0; Flag; dwBus++)
	{
		for (dwDeviceNumber = 0; dwDeviceNumber < PCI_MAX_DEVICES && Flag; dwDeviceNumber++)
		{
			pSlotNumber->u.bits.DeviceNumber = dwDeviceNumber;

			for (dwFunctionNumber = 0; dwFunctionNumber < PCI_MAX_FUNCTION; dwFunctionNumber++)
			{
				pSlotNumber->u.bits.FunctionNumber = dwFunctionNumber;

				dwBytesWritten = RtGetBusDataByOffset(
					PCIConfiguration,
					dwBus,
					pSlotNumber->u.AsULONG,
					PciData,
					0,
					PCI_COMMON_HDR_LENGTH
					);

				if (dwBytesWritten == 0)
				{
					// out of PCI buses 
					Flag = FALSE;
					break;
				}

				if (dwBytesWritten == 2 && PciData->VendorID == PCI_INVALID_VENDORID)
				{
					// no device at this slot number, skip to next slot 
					break;
				}

				//
				// A device is found, if this is our card (Biss Pro 1000 PT)
				//  return, otherwise continue until all buses have been searched.

				if ((PciData->VendorID == 0x1172) && (PciData->DeviceID == 0x4258))
					if (nCardIndex == nCardNumber) // This is our card!!
						return dwBus;   // stop searching
					else
						nCardIndex++;
			} // FunctionNumber-loop

		} // DeviceNumber-loop

	}  // Bus-loop
	return -1;
}

bool CBissCard::InitDeviceBiss(PPCI_COMMON_CONFIG PciData, PCI_SLOT_NUMBER *pSlotNumber, int nBusNum)
{

	LARGE_INTEGER memAddr;		// a base port address 
	LARGE_INTEGER tranMemAddr;	// translated base addresses returned by RtMapMemory
	ULONG AddressSpace = 0;     // indicates memory space
	ULONG bytesWritten = 0;
	int i = 0;
	int addressRange = 4 * 1024;

	//
	// Get the virtual bus addresses for each address register 
	// if call fails set to NULL and continue
	//
	for (i = 0; i< 3; i++)
	{
		memAddr.QuadPart = PciData->u.type0.BaseAddresses[i];
		m_memAddr[i] = memAddr;
		//
		// Translate the base port addresses to system mapped addresses. 
		//
		//RtPrintf("PCIe Base Addr %d:%X\n", i, memAddr.QuadPart);
		if (!RtTranslateBusAddress(
			PCIBus,			// bus interface type
			m_dwBusNum,		// bus number (zero based)
			memAddr,		// bus-relative address
			&AddressSpace,	// specifies port number or a memory address
			&tranMemAddr	// pointer to the translated address
			))
		{
			vMemAddr[i] = NULL;
			continue;
		}
		//RtPrintf("tranMemAddr LOW %X\n", tranMemAddr.u.LowPart);
		//RtPrintf("tranMemAddr High %X\n", tranMemAddr.u.HighPart);

		//
		// Map the addresses to virtual addresses the software can use
		//
		vMemAddr[i] = (PUCHAR)RtMapMemory(
			tranMemAddr,	// base of the physical address range to map
			addressRange,	// the length of address range in bytes
			MmNonCached				// whether or not to use cache
			);
		//RtPrintf("vMemAddr%d:%X\n", i, vMemAddr[i]);
	}

	if ((vMemAddr[1] == NULL) || (vMemAddr[2] == NULL))
		return false;
	//
	// Set the command parameter so we can access the PCI device's control registers.
	// 
	PciData->Command = (PCI_ENABLE_IO_SPACE | PCI_ENABLE_MEMORY_SPACE |
		PCI_ENABLE_BUS_MASTER | PCI_ENABLE_WRITE_AND_INVALIDATE);


	bytesWritten = RtSetBusDataByOffset(
		PCIConfiguration,		// type of bus data to be set
		m_dwBusNum,				// bus number (zero based)
		pSlotNumber->u.AsULONG,	// logical slot number
		PciData,				// pointer buffer containing configuration information
		0,						// byte offset in the buffer 
		PCI_COMMON_HDR_LENGTH	// number of bytes in the buffer
		);

	if (bytesWritten == 0)
		return false;
	return true;


}

void CBissCard::SetPcieClock(ULONG clock)
{
	//RtWritePortUlong((PULONG)(m_memAddr[1].QuadPart + 0x4C), clock);
	m_clock_freq = clock;
}

void CBissCard::SetProtocol(ULONG prot)
{
	//RtWritePortUlong((PULONG)(m_memAddr[1].QuadPart + 0x2C), prot);
	m_protocol = prot;
}

bool CBissCard::SetChannelBit(int Chn, unsigned long bitcount)
{
	if (Chn<0 || Chn>4)
		return false;
	if (bitcount == 0 || bitcount > 32)
		return false;
	//RtWritePortUlong((PULONG)(m_memAddr[1].QuadPart + 0x18 + Chn*4), bitcount);
	m_clock_bit[Chn] = bitcount;
	return true;
}

int CBissCard::ConfigDeviceBiss()
{
	m_llMemoryAddr.u.HighPart = 0x0;
	//m_llMemoryAddr.u.LowPart = 0xffff + 1024 * 8 * m_dwCardNum;
	m_llMemoryAddr.u.LowPart = 0xffffffff;
	m_pdwPhyMemory = (PULONG)RtAllocateContiguousMemory(1024 * 8, m_llMemoryAddr);
	if (m_pdwPhyMemory == NULL)
	{
		RtPrintf("RtAllocateContiguousMemory failed.\n");
		return -1;
		//ExitProcess(0);
	}
	m_llMemoryAddr = RtGetPhysicalAddress(m_pdwPhyMemory);
	if (m_llMemoryAddr.u.HighPart != 0x0)
	{
		RtPrintf("Memory HighPart Value Error.\n");
		return -2;
		//ExitProcess(0);
	}
	return 1;
	//m_llMemoryAddr.u.HighPart = 0x0;
	//RtPrintf("m_llMemoryAddr:%X\n", m_llMemoryAddr);
	//RtPrintf("m_llMemoryAddr HighPart:%X\n", m_llMemoryAddr.u.HighPart);
	//RtPrintf("m_llMemoryAddr LowPart:%X\n", m_llMemoryAddr.u.LowPart);

	//m_llMemoryAddr.u.HighPart = 0x0;
	//m_pdwPhyMemory = (PULONG)RtAllocateContiguousMemory(1024 * 8, m_llMemoryAddr);
	//if (m_pdwPhyMemory == NULL)
	//{
	//	RtPrintf("RtAllocateContiguousMemory failed.\n");
	//}
	//m_llMemoryAddr = RtGetPhysicalAddress(m_pdwPhyMemory);
}

void CBissCard::Snc_CollectBiss()
{
	//ֹͣ
	*(PULONG)(vMemAddr[1] + 0x48) = 0;

	//FIFO
	*(PULONG)(vMemAddr[1] + 0x10) = 0x80000000;
	*(PULONG)(vMemAddr[1] + 0x10) = 0;

	//дdma׵ַ
	*(PULONG)(vMemAddr[1] + 0x8) = m_llMemoryAddr.u.LowPart;
	//дdmaֽ
	*(PULONG)(vMemAddr[1] + 0xC) = 5 * 8 * m_iDataGroupNum;//5ͨ8byteǰΪ5*8*3
														   //dma䷽'
	*(PULONG)(vMemAddr[1] + 0x10) = 0x100;
	//Э
	*(PULONG)(vMemAddr[1] + 0x2C) = m_protocol;
	//ʱ
	*(PULONG)(vMemAddr[1] + 0x4C) = m_clock_freq;
	//λ
	for (int i = 0; i < 5; i++)
	{
		*(PULONG)(vMemAddr[1] + 0x18 + i * 4) = m_clock_bit[i];
	}

	//ж
	*(PULONG)(vMemAddr[1] + 0x4) = 0x2;

	//ʼ
	*(PULONG)(vMemAddr[1] + 0x48) = 1;//

}

int CBissCard::DMA_TransmissionEnd()
{
	int i, j;
	j = 0;
	//Snc_CollectBiss();

	while (((*(PULONG)(vMemAddr[1] + 0x4)) & 1) == 0)
	{
		//__asm { nop };
		j++;
		if (j>500)
		{
			//RtPrintf("ad\n");

			return 0;
			//break;
		}

	};

	//	if ((*(PLONG)(vMemAddr[1]+4)) == 3)

	*(PULONG)(vMemAddr[1] + 0x48) = 0;
	{
		i = 0;
		for (int index = m_iDataGroupNum - 1; index > -1; index--)
		{
			m_snrdat.a.low = m_pdwPhyMemory[i++];
			m_snrdat.a.high = m_pdwPhyMemory[i++];
			memcpy(&m_ppsBissInfo[index][0], &m_snrdat, sizeof(m_snrdat));
			m_snrdat.a.low = m_pdwPhyMemory[i++];
			m_snrdat.a.high = m_pdwPhyMemory[i++];
			memcpy(&m_ppsBissInfo[index][1], &m_snrdat, sizeof(m_snrdat));
			m_snrdat.a.low = m_pdwPhyMemory[i++];
			m_snrdat.a.high = m_pdwPhyMemory[i++];
			memcpy(&m_ppsBissInfo[index][2], &m_snrdat, sizeof(m_snrdat));
			m_snrdat.a.low = m_pdwPhyMemory[i++];
			m_snrdat.a.high = m_pdwPhyMemory[i++];
			memcpy(&m_ppsBissInfo[index][3], &m_snrdat, sizeof(m_snrdat));
			m_snrdat.a.low = m_pdwPhyMemory[i++];
			m_snrdat.a.high = m_pdwPhyMemory[i++];
			memcpy(&m_ppsBissInfo[index][4], &m_snrdat, sizeof(m_snrdat));

		}
	}
	return 1;

}
void CBissCard::SetDataGroupNum(int iDataGroupNum)
{
	m_iDataGroupNum = iDataGroupNum;
	if (m_ppsBissInfo != NULL)
	{
		delete[]m_ppsBissInfo;
	}
	m_ppsBissInfo = new __PACKET*[m_iDataGroupNum];
	for (int i = 0; i < m_iDataGroupNum; i++)
	{
		m_ppsBissInfo[i] = new __PACKET[5];
	}
}
int CBissCard::GetCurrentSpeed(double& dCurSpeed, int iChannelNum, bool bIsNeedEWcheck, bool bIsAngle, double dLineRes)
{
	if (m_iDataGroupNum < 2)
	{
		return -1;
	}
	if (m_clock_bit[iChannelNum] < 1 || m_clock_bit[iChannelNum]>32)
		return -2;
	if (bIsNeedEWcheck)
	{
		if (m_ppsBissInfo[0][iChannelNum].b.err
			&&	m_ppsBissInfo[0][iChannelNum].b.war
			&&	m_ppsBissInfo[1][iChannelNum].b.err
			&&	m_ppsBissInfo[1][iChannelNum].b.war)
		{
			if (bIsAngle)
			{
				double LocDiff = 360.00*(m_ppsBissInfo[0][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum]))) / (0xffffffff >> (32 - m_clock_bit[iChannelNum]))
					- 360.00*(m_ppsBissInfo[1][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum]))) / (0xffffffff >> (32 - m_clock_bit[iChannelNum]));
				double TimeDiff = (m_ppsBissInfo[0][iChannelNum].b.div & 0xffff)*1.0*(1e-8) - (m_ppsBissInfo[1][iChannelNum].b.div & 0xffff)*1.0*(1e-8);
				if (LocDiff > 300)
				{
					LocDiff -= 360;
				}
				else if (LocDiff < -300)
				{
					LocDiff += 360;
				}
				if (TimeDiff > 50000 * (1e-8))
				{
					TimeDiff = 65536 * (1e-8) - TimeDiff;
				}
				else if (TimeDiff < -50000 * (1e-8))
				{
					TimeDiff += 65536 * (1e-8);
				}
				dCurSpeed = LocDiff / TimeDiff;
				return 1;
			}
			else
			{
				double LocDiff = 1000.0*dLineRes*(m_ppsBissInfo[0][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum])))
					- 1000.0*dLineRes*(m_ppsBissInfo[1][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum])));
				double TimeDiff = (m_ppsBissInfo[0][iChannelNum].b.div & 0xffff)*1.0*(1e-8) - (m_ppsBissInfo[1][iChannelNum].b.div & 0xffff)*1.0*(1e-8);
				if (TimeDiff > 50000 * (1e-8))
				{
					TimeDiff = 65536 * (1e-8) - TimeDiff;
				}
				else if (TimeDiff < -50000 * (1e-8))
				{
					TimeDiff += 65536 * (1e-8);
				}
				dCurSpeed = LocDiff / TimeDiff;
				return 1;

			}
		}
		else
			return 0;
	}
	else
	{
		if (bIsAngle)
		{
			double LocDiff = 360.00*(m_ppsBissInfo[0][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum]))) / (0xffffffff >> (32 - m_clock_bit[iChannelNum]))
				- 360.00*(m_ppsBissInfo[1][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum]))) / (0xffffffff >> (32 - m_clock_bit[iChannelNum]));
			double TimeDiff = (m_ppsBissInfo[0][iChannelNum].b.div & 0xffff)*1.0*(1e-8) - (m_ppsBissInfo[1][iChannelNum].b.div & 0xffff)*1.0*(1e-8);
			if (LocDiff > 300)
			{
				LocDiff -= 360;
			}
			else if (LocDiff < -300)
			{
				LocDiff += 360;
			}
			if (TimeDiff > 50000 * (1e-8))
			{
				TimeDiff = 65536 * (1e-8) - TimeDiff;
			}
			else if (TimeDiff < -50000 * (1e-8))
			{
				TimeDiff += 65536 * (1e-8);
			}
			dCurSpeed = LocDiff / TimeDiff;
			return 1;
		}
		else
		{
			double LocDiff = 1000.0*dLineRes*(m_ppsBissInfo[0][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum])))
				- 1000.0*dLineRes*(m_ppsBissInfo[1][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum])));
			double TimeDiff = (m_ppsBissInfo[0][iChannelNum].b.div & 0xffff)*1.0*(1e-8) - (m_ppsBissInfo[1][iChannelNum].b.div & 0xffff)*1.0*(1e-8);
			if (TimeDiff > 50000 * (1e-8))
			{
				TimeDiff = 65536 * (1e-8) - TimeDiff;
			}
			else if (TimeDiff < -50000 * (1e-8))
			{
				TimeDiff += 65536 * (1e-8);
			}
			dCurSpeed = LocDiff / TimeDiff;
			return 1;
		}
	}
	return 0;


}
int CBissCard::GetCurrentLocation(double& dCurLocation, int iChannelNum, bool bIsNeedEWcheck, bool bIsAngle, double dLineRes)
{
	if (m_clock_bit[iChannelNum] < 1 || m_clock_bit[iChannelNum]>32)
		return -2;
	if (bIsNeedEWcheck)
	{
		if (bIsAngle)
		{
			for (int i = 0; i < m_iDataGroupNum; i++)
			{
				if (m_ppsBissInfo[i][iChannelNum].b.err
					&&	m_ppsBissInfo[i][iChannelNum].b.war)
				{
					dCurLocation = 360.00*(m_ppsBissInfo[i][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum]))) / (0xffffffff >> (32 - m_clock_bit[iChannelNum]));
					return i;
				}
			}
			return -1;
		}
		else
		{
			for (int i = 0; i < m_iDataGroupNum; i++)
			{
				if (m_ppsBissInfo[i][iChannelNum].b.err
					&&	m_ppsBissInfo[i][iChannelNum].b.war)
				{
					dCurLocation = 1000.0*dLineRes*(m_ppsBissInfo[i][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum])));
					return i;
				}
			}
			return -1;
		}

	}
	else
	{
		if (bIsAngle)
		{

			dCurLocation = 360.00*(m_ppsBissInfo[0][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum]))) / (0xffffffff >> (32 - m_clock_bit[iChannelNum]));
			return 0;
		}
		else
		{

			dCurLocation = 1000.0*dLineRes*(m_ppsBissInfo[0][iChannelNum].b.data & (0xffffffff >> (32 - m_clock_bit[iChannelNum])));
			return 0;
		}
	}

	return -1;
}
void CBissCard::CleanupDeviceBiss()
{
	*(PULONG)(vMemAddr[1] + 0x48) = 0;
	for (int i = 0; i < 3; i++)
	{
		if (vMemAddr[i] != NULL)
		{
			RtUnmapMemory((PVOID)vMemAddr[i]);
		}
	}
	if (m_pdwPhyMemory != NULL)
	{
		RtFreeContiguousMemory(m_pdwPhyMemory);
	}
}